// name: renderer.h
// desc: this is the renderer

#include "renderer.h"
#include "mo_audio.h"
#include "mo_gfx.h"
#import "mo_fun.h"
#include <vector>

#import "mo_touch.h"
#import "mo_fun.h"
#import "Plucked.h"
#import "Shakers.h"

// defines
#define SRATE 24000
#define FRAMESIZE 512
#define NUM_CHANNELS 2

#define MAX_TOUCHES 3
#define MAX_BOUNCERS 100

// globals
GLuint g_texture[5];
SAMPLE g_width = 2.5;
float g_fc = 440.0;
class Projectile;
Projectile * g_bouncers[MAX_BOUNCERS];
float buildingScale = .5;
int maxCollisions = 30;

class TouchEntity;
TouchEntity * g_touches[MAX_TOUCHES];
GLuint g_numActiveTouches = 0;

// graphics
GLint g_gfxWdith = 0;
GLint g_gfxHeight = 0;

// buffer
SAMPLE g_vertices[FRAMESIZE*2];


GLfloat rand2f( float a, float b )
{
    GLfloat diff = b - a;
    return a + ((GLfloat)rand() / RAND_MAX)*diff;
}

using namespace stk;

Plucked * pluckah;
Shakers * drummah;


// draw a touch entity
void TouchEntity::render()
{
    if( !active ) return; // touch in g_touches is empty
    
	static const GLfloat half_width = 30;
    static const GLfloat squareVertices[] = {
        -half_width, -half_width,
        half_width, -half_width,
        -half_width, half_width,
        half_width, half_width,
    };
	
	static const GLfloat normals[] = {
        0, 0, 1,
        0, 0, 1,
        0, 0, 1,
        0, 0, 1
    };
    
    static const GLfloat texCoords[] = {
        0, 1,
        1, 1,
        0, 0,
        1, 0
    };
	
	
	///////////////////////
    // enable texture mapping
    glEnable( GL_TEXTURE_2D );
    // enable blending
    glEnable( GL_BLEND );
    // blend function
    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
    
    glPushMatrix();
    
    // enable
    glEnableClientState( GL_VERTEX_ARRAY );
    glEnableClientState( GL_NORMAL_ARRAY );
    glEnableClientState( GL_TEXTURE_COORD_ARRAY );
	
    // color
    glColor4f( col.x, col.y, col.z, alpha );
    // vertex
    glVertexPointer( 2, GL_FLOAT, 0, squareVertices );
    // normal
    glNormalPointer( GL_FLOAT, 0, normals );
    // texture coordinate
    glTexCoordPointer( 2, GL_FLOAT, 0, texCoords );
    
	// rotate
	glRotatef( ori.x, 0, 0, 1 );
	
	// scale
	glScalef( sca.x * buildingScale, sca.y * buildingScale, sca.z * buildingScale );
	
	glBindTexture( GL_TEXTURE_2D, g_texture[3] );
	//NSLog( @"textureNum is %d", textureNum);
	
    // triangle strip
    glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );
    
    // disable
    glDisableClientState( GL_VERTEX_ARRAY );
    glDisableClientState( GL_NORMAL_ARRAY );
    glDisableClientState( GL_TEXTURE_COORD_ARRAY );
    
    glPopMatrix();
    
    // disable
    glDisable( GL_TEXTURE_2D );
    glDisable( GL_BLEND );
	/////////////////////////////
	
    glPushMatrix();
    // color
    glColor4f( col.x, col.y, col.z, alpha );
    // vertex
    glVertexPointer( 2, GL_FLOAT, 0, squareVertices );
    
    // triangle strip
    glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );
    glPopMatrix();
	
}

// draw a touch entity
void Projectile::render()
{
    if( !active ) return; // touch in g_touches is empty
    
	static const GLfloat half_width = 30;
    static const GLfloat squareVertices[] = {
        -half_width, -half_width,
        half_width, -half_width,
        -half_width, half_width,
        half_width, half_width,
    };
	
	static const GLfloat normals[] = {
        0, 0, 1,
        0, 0, 1,
        0, 0, 1,
        0, 0, 1
    };
    
    static const GLfloat texCoords[] = {
        0, 1,
        1, 1,
        0, 0,
        1, 0
    };
	
	
	///////////////////////
    // enable texture mapping
    glEnable( GL_TEXTURE_2D );
    // enable blending
    glEnable( GL_BLEND );
    // blend function
    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
    
    glPushMatrix();
    
    // enable
    glEnableClientState( GL_VERTEX_ARRAY );
    glEnableClientState( GL_NORMAL_ARRAY );
    glEnableClientState( GL_TEXTURE_COORD_ARRAY );
	
    // color
    glColor4f( col.x, col.y, col.z, alpha );
    // vertex
    glVertexPointer( 2, GL_FLOAT, 0, squareVertices );
    // normal
    glNormalPointer( GL_FLOAT, 0, normals );
    // texture coordinate
    glTexCoordPointer( 2, GL_FLOAT, 0, texCoords );
    
	// rotate
	//NSLog( @"bouncer has xRotation %f", xRotation);
	ori.x += xRotation;
	glRotatef( ori.x, 0, 0, 1 );

	
	// scale
	glScalef( sca.x * buildingScale, sca.y * buildingScale, sca.z * buildingScale );
	
	glBindTexture( GL_TEXTURE_2D, g_texture[textureNum] );
	//NSLog( @"textureNum: %d", textureNum );
	
    // triangle strip
    glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );
    
    // disable
    glDisableClientState( GL_VERTEX_ARRAY );
    glDisableClientState( GL_NORMAL_ARRAY );
    glDisableClientState( GL_TEXTURE_COORD_ARRAY );
    
    glPopMatrix();
    
    // disable
    glDisable( GL_TEXTURE_2D );
    glDisable( GL_BLEND );
	/////////////////////////////
	
    glPushMatrix();
    // color
    glColor4f( col.x, col.y, col.z, alpha );
    // vertex
    glVertexPointer( 2, GL_FLOAT, 0, squareVertices );
    
    // triangle strip
    glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );
    glPopMatrix();
	
	
	// just to draw flames (consolidate later!)
	///////////////////////////////////////////////
	//NSLog( @"if %d > %d", collisionCount ,(2 * MAX_COLLISIONS / 3));
	if ( collisionCount >= (3 * maxCollisions / 5)) {

		// enable texture mapping
		glEnable( GL_TEXTURE_2D );
		// enable blending
		glEnable( GL_BLEND );
		// blend function
		glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
		
		glPushMatrix();
		
		// enable
		glEnableClientState( GL_VERTEX_ARRAY );
		glEnableClientState( GL_NORMAL_ARRAY );
		glEnableClientState( GL_TEXTURE_COORD_ARRAY );
		
		// color
		glColor4f( col.x, col.y, col.z, alpha );
		// vertex
		glVertexPointer( 2, GL_FLOAT, 0, squareVertices );
		// normal
		glNormalPointer( GL_FLOAT, 0, normals );
		// texture coordinate
		glTexCoordPointer( 2, GL_FLOAT, 0, texCoords );
		
		// rotate
		//NSLog( @"bouncer has xRotation %f", xRotation);
		ori.x += xRotation;
		glRotatef( ori.x, 0, 0, 1 );
		
		// scale
		glScalef( 1.5 * buildingScale,1.5 * buildingScale,1.5 * buildingScale ); // numbers hacked by hand - fix later
		
		glBindTexture( GL_TEXTURE_2D, g_texture[4] );
		
		// triangle strip
		glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );
		
		// disable
		glDisableClientState( GL_VERTEX_ARRAY );
		glDisableClientState( GL_NORMAL_ARRAY );
		glDisableClientState( GL_TEXTURE_COORD_ARRAY );
		
		glPopMatrix();
		
		// disable
		glDisable( GL_TEXTURE_2D );
		glDisable( GL_BLEND );
		
		
		glPushMatrix();
		// color
		glColor4f( col.x, col.y, col.z, alpha );
		// vertex
		glVertexPointer( 2, GL_FLOAT, 0, squareVertices );
		
		// triangle strip
		glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );
		glPopMatrix();
	}
	
}

int Projectile::updatePosition()
{
	// 480 by 320
	//radiusCheck = 1;
	int collision = 0;
	
	loc = loc + vel;
	
	if ( loc.x > 480 ) {
		vel.x = -vel.x;
		loc.x = 480 - (loc.x - 480);
		collision = 1;
	}
	if ( loc.x < 0 ) {
		vel.x = -vel.x;
		loc.x = 0 + (0 - loc.x);
		collision = 1;
	}
	if ( loc.y > 320 ) {
		vel.y = -vel.y;
		loc.y = 320 - (loc.y - 320);
		collision = 1;
	}
	if ( loc.y < 0 ) {
		vel.y = -vel.y;
		loc.y = 0 + ( 0 - loc.y );
		collision = 1;
	}
	return collision;
};

// touch callback
void touchCallback( NSSet * touches, UIView * view, const std::vector<MoTouchTrack> & touchPts, void * data)
{
    // iterate over touch points
    CGPoint location;
    for( UITouch * touch in touches )
    {
        // get the location
        location = [touch locationInView:nil];
		
		// transform: to make landscape
        double temp = location.x;
        location.x = location.y;
        location.y = temp;
		
		// NSLog( @"touch: %f, %f,", location.x, location.y );
		
        if( touch.phase == UITouchPhaseBegan )
        {
            // find idle touch entity
            TouchEntity * entity = NULL;
            for( int i = 0; i < MAX_TOUCHES; i++ )
            {
				// if it's the third touch
				if ( i == MAX_TOUCHES - 1 ) {
					g_touches[i]->textureNum = rand() % 4;
					NSLog( @"touch number %d",i);
				}
				
				
                // in case touch already active
                if( g_touches[i]->touch_ref == touch )
                    break;
                
                // find the next non-active touch
                if( !g_touches[i]->active )
                {
                    entity = g_touches[i];
                    break;
                }
            }
            
            // sanity check
            if( entity != NULL )
            {
                // set
                entity->active = true;
                entity->touch_ref = touch;
                entity->loc.x = location.x;
                entity->loc.y = location.y;  
				entity->ori.x = rand2f(0, 180);
                // count
                g_numActiveTouches++;
                // log it
                // NSLog( @"active touches: %d", g_numActiveTouches );
            }
        }
        else if( touch.phase == UITouchPhaseMoved )
        {
            for( int i = 0; i < MAX_TOUCHES; i++ )
            {
                if( g_touches[i]->touch_ref == touch )
                {
                    g_touches[i]->loc.x = location.x;
                    g_touches[i]->loc.y = location.y;
                    break;
                }
            }
        }
        else if( (touch.phase == UITouchPhaseEnded) || (touch.phase == UITouchPhaseCancelled) )
        {
            for( int i = 0; i < MAX_TOUCHES; i++ )
            {
                if( g_touches[i]->touch_ref == touch )
                {
					// 3rd touch is lifted!
					if ( i == 2 ) {
						
						int j;
						for ( j = 0; j < MAX_BOUNCERS; j++ ) {
							// NSLog( @"bouncer %d is %d", j, g_bouncers[j]->active? 1 : 0 );
							
							if ( !g_bouncers[j]->active ) {
								break;
							}
						}
						
						// ZEN FIRE ZE MISSILES!
						if (j < MAX_BOUNCERS) {
							
							// NSLog( @"Zoinks! Moving bouncers[%d] to %f, %f", j, g_bouncers[j]->loc.x, g_bouncers[j]->loc.y );
							g_bouncers[j]->active = true;
							g_bouncers[j]->loc.x = g_touches[i]->loc.x;
							g_bouncers[j]->loc.y = g_touches[i]->loc.y;
							
							
							// midpoint between anchors
							Vector3D mid;
							mid = g_touches[0]->loc + g_touches[1]->loc ;
							mid = mid * .5;
							
							//NSLog( @"middle of (%f,%f) and (%f,%f) is (%f,%f)", g_touches[0]->loc.x,g_touches[0]->loc.y, g_touches[1]->loc.x,g_touches[1]->loc.y, mid.x,mid.y);
							
							// set the velocity based on magnitude of pull
							Vector3D pullVec = mid - g_bouncers[j]->loc ;
							
							g_bouncers[j]->vel = pullVec * .05 ;
							
							
							//
							//NSLog( @"touch texture: %d, ori.x: %f", g_touches[2]->textureNum, g_touches[2]->ori.x);
							g_bouncers[j]->textureNum = g_touches[2]->textureNum	;
							g_bouncers[j]->rad = 25 + g_bouncers[j]->textureNum * 10;
							g_bouncers[j]->ori.z = g_touches[2]->ori.z;
							//NSLog( @"bounder texture: %d, ori.x: %f, rad: %d", g_bouncers[j]->textureNum, g_bouncers[j]->ori.x, g_bouncers[j]->rad);
							
							
							pluckah->pluck( 1.0 ); // TODO: change velocity based on magnitude of pull
						}
					
						
					}
                    // set
                    g_touches[i]->active = false;
                    g_touches[i]->touch_ref = NULL;
                    
                    // pack active touches
                    for( int j = i+1; j < MAX_TOUCHES; j++ )
                    {
                        if( g_touches[j]->active )
                        {
                            // swap
                            TouchEntity * swap = g_touches[i];
                            g_touches[i] = g_touches[j];
                            g_touches[j] = swap;
                            // dangerous: advance i
                            i = j;
                        }
                    }
                    
                    // count
                    g_numActiveTouches--;
                    // log
                    // NSLog( @"active touches: %d", g_numActiveTouches );
                    
                    break;
                }
            }
        }
    }
}
///////////// end touch stuff


// audio callback
void audio_callback( Float32 * buffer, UInt32 numFrames, void * userData )
{
	// SAMPLE x = 0;
    // SAMPLE inc = g_width / numFrames;
    
	/*
    for( int i = 0; i < numFrames; i++ )
    {
        // x coordinate
        g_vertices[2*i] = x; x += inc;
        // y coordinate
        g_vertices[2*i+1] = buffer[2*i] * 3; // 3 is arbitrary vertical scaling
        // zero
        buffer[2*i] = buffer[2*i+1] = 0;
		
    }
	*/
	 
	for ( int i = 0; i < numFrames; i++ )
	{
		pluckah->setFrequency(g_fc);
		buffer[2*i] =  pluckah->tick();
		buffer[2*i+1] = pluckah->tick();
		buffer[2*i] += drummah->tick();
		buffer[2*i+1] += drummah->tick();
	}
}


// initialize
bool init()
{
	
	// init graphics
    glClearColor( .5, .5, .5, 1 );
    // line width
    glLineWidth( 2.0 );
    // generate texture name
    glGenTextures( 4, &g_texture[0] );
	
    // bind the texture
    glBindTexture( GL_TEXTURE_2D, g_texture[0] );
    // setting parameters
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    // load the texture
    MoGfx::loadTexture( @"Supply256", @"png" );
	
	// bind the texture
	glBindTexture( GL_TEXTURE_2D, g_texture[1] );
    // setting parameters
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    // load the texture
    MoGfx::loadTexture( @"Armory256", @"png" );
	
	// bind the texture
	glBindTexture( GL_TEXTURE_2D, g_texture[2] );
    // setting parameters	
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    // load the texture
    MoGfx::loadTexture( @"Starport256", @"png" );
	
	glBindTexture( GL_TEXTURE_2D, g_texture[3] );
    // setting parameters
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    // load the texture
    MoGfx::loadTexture( @"Command256", @"png" );
	
	
	glBindTexture( GL_TEXTURE_2D, g_texture[4] );
    // setting parameters
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    // load the texture
    MoGfx::loadTexture( @"flames256", @"png" );
	
	 
	pluckah	= new Plucked(200.0);
	drummah	= new Shakers();
	
	MoTouch::addCallback( touchCallback, NULL );
	
	for (int i = 0; i < MAX_BOUNCERS; i++) {
		g_bouncers[i] = new Projectile();
		g_bouncers[i]->col.set( 1.0, 1.0, 1.0 );
		g_bouncers[i]->sca.set( 2.0, 2.0, 2.0 );
		g_bouncers[i]->textureNum = 1;
		g_bouncers[i]->rad = 30.0;
		g_bouncers[i]->ori.x = rand2f(0, 180);
		g_bouncers[i]->xRotation = rand2f(0, 4) - 2;
		
		g_bouncers[i]->loc.set( 150.0, 150.0, 0.0 );
		g_bouncers[i]->vel.set( 1.0, 1.0, 0.0 );
	}
	
	for (int i = 0; i < MAX_TOUCHES; i++) {
		g_touches[i] = new TouchEntity();
		g_touches[i]->col.set( 1.0, 1.0, 1.0 );
		g_touches[i]->sca.set( 2.0, 2.0, 2.0 );
		g_touches[i]->textureNum = 3;
		g_bouncers[i]->ori.x = rand2f(0, 180);
	}
	
	
	// init
    bool result = MoAudio::init( SRATE, FRAMESIZE, NUM_CHANNELS );
    if( !result )
    {
        // do not do this:
        int * p = 0;
        *p = 0;
    }
    // start
    result = MoAudio::start( audio_callback, NULL );
    if( !result )
    {
        // do not do this:
        int * p = 0;
        *p = 0;
    }
	
	return true;
}


// set graphics dimensions
void setDims( GLint width, GLint height )
{
	g_gfxWdith = width;
	g_gfxHeight = height;
}


// clean up
void cleanup()
{
	// Do me laterz
	delete pluckah;
	delete drummah;
}


// helper: renders each of the touch entities
void drawEntities()
{
	for( int i = 0; i < MAX_TOUCHES; i++ )
    {
        glPushMatrix();
        // translate
        glTranslatef( g_touches[i]->loc.x, g_touches[i]->loc.y, g_touches[i]->loc.z );
        // render
        g_touches[i]->render();
        
        glPopMatrix();
    }
	
	for( int i = 0; i < MAX_BOUNCERS; i++ )
    {
        glPushMatrix();
        // translate
        glTranslatef( g_bouncers[i]->loc.x, g_bouncers[i]->loc.y, g_bouncers[i]->loc.z );
        // render
		
		g_bouncers[i]->render();
        
        glPopMatrix();
    }
}


// helper: draws a line segment between the first two touches
void drawLineBetweenTouches()
{
	float x0;
	float y0;
	float x1;
	float y1;
	float x2;
	float y2;
	
	if( g_numActiveTouches < 2 )
	{
		return;
	}
	else if ( g_numActiveTouches == 2 )
	{
		assert( g_touches[0]->active && g_touches[1]->active );
		
		x0 = g_touches[0]->loc.x;
		y0 = g_touches[0]->loc.y;
		x1 = g_touches[1]->loc.x;
		y1 = g_touches[1]->loc.y;
		x2 = g_touches[1]->loc.x;
		y2 = g_touches[1]->loc.y;
		
		
		// set pitch based on distance
		g_fc = 3 * ( g_touches[0]->loc - g_touches[1]->loc ).magnitude();

	}
	else if ( g_numActiveTouches == 3 )
	{
		assert( g_touches[0]->active && g_touches[1]->active && g_touches[2]->active);
		
		x0 = g_touches[0]->loc.x;
		y0 = g_touches[0]->loc.y;
		x1 = g_touches[1]->loc.x;
		y1 = g_touches[1]->loc.y;
		x2 = g_touches[2]->loc.x;
		y2 = g_touches[2]->loc.y;
		
	}
	
	
	// draw straight line from v0 to v1 to v2
	GLfloat lineVertices1[4] = {x0, y0, x2, y2};
	GLfloat lineVertices2[4] = {x1, y1, x2, y2};
	
	glPushMatrix();
	
	glVertexPointer(2, GL_FLOAT, 0, lineVertices1);
	glVertexPointer(2, GL_FLOAT, 0, lineVertices2);
	
	glEnableClientState( GL_VERTEX_ARRAY );
	glColor4f( 0.0f, 0.7f, 0.0f, 0.0f ); // red line
	glLineWidth(2.0);					 // thick line
	
	glDrawArrays( GL_LINE_STRIP, 0, 3 ); 
	glPopMatrix();
}



int calculateBallCollisions( Projectile * myBall, int ballNum )
{
	int retVal = 0;
	for ( int i = 0 ; i < MAX_BOUNCERS; i++ ) {
		
		if ( g_bouncers[i]->active ) {
			
			Projectile * yourBall = g_bouncers[i];
			
			if ( myBall != yourBall ) {
				// NSLog( @"balls %d and %d not equal", ballNum, i );
				
				// 1. find distance dist between myBall and yourBall
				float dist = ( yourBall->loc - myBall->loc ).magnitude();
				// NSLog( @"Distance between myball %d, and yourball %d is %f", ballNum, i, dist );
				
				// 2. if d < b.rad + i.rad , then the two balls hit!
				if ( ( myBall->rad * buildingScale + yourBall->rad * buildingScale ) > dist ) {
					myBall->collisionCount++;
					yourBall->collisionCount++;
					playCollisionSound();
					
					// NSLog( @" Balls %d and %d collide!!!", ballNum, i );
					
					Vector3D positionDifference = yourBall->loc - myBall->loc;
					//NSLog( @"position difference: (%f, %f, %f)", positionDifference.x, positionDifference.y, positionDifference.z );
					positionDifference.normalize();
					
					Vector3D myBallVelocityProjected = positionDifference * (myBall->vel * positionDifference);
					Vector3D yourBallVelocityProjected = (positionDifference * -1) * (yourBall->vel * (positionDifference * -1));
					
					myBall->vel -= myBallVelocityProjected;
					yourBall->vel -= yourBallVelocityProjected;
					
					myBall->vel += yourBallVelocityProjected;
					yourBall->vel += myBallVelocityProjected;
					
					yourBall->loc = myBall->loc + positionDifference * (myBall->rad * buildingScale + yourBall->rad * buildingScale + 1);
					//NSLog( @"position difference: (%f, %f, %f)", positionDifference.x, positionDifference.y, positionDifference.z );
				}
			}
		}
	}
	return retVal;
}

void playCollisionSound()
{
	drummah->noteOn(1,.9);
}

void setBuildingScale( float size )
{
	buildingScale = size;
}

void setCollisionNum( float collisionz )
{
	maxCollisions = (int)(collisionz);

}

// render
void render()
{
	// reset projection matrix
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrthof( -1.0f, 1.0f, -1.5f, 1.5f, -1.0f, 1.0f );
	// modelview!
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	
	// set clear color
	glClearColor( 0, 0, 0, 0.5f );
	glClear( GL_COLOR_BUFFER_BIT );
	
	
	// map the viewport
	MoGfx::ortho( 320, 480, 1 ); // 1 is for landscape
	
	glEnableClientState( GL_VERTEX_ARRAY );
	
	for ( int i = 0; i < MAX_BOUNCERS; i++) {
		if ( g_bouncers[i]->active ) {

			// calculate ball collisions
			int deflect = calculateBallCollisions( g_bouncers[i] , i );
			
			// move balls in their proper directions
			int bounce = g_bouncers[i]->updatePosition();
			
			// add collisions to the ball's collision count
			if ( bounce || deflect ) {
				playCollisionSound();
				if ( bounce ) {
					g_bouncers[i]->collisionCount ++;
					//NSLog( @"bounce!");
				}
				if ( deflect ) {
					NSLog( @"deflect in! %d", g_bouncers[i]->collisionCount);
					g_bouncers[i]->collisionCount ++;
					NSLog( @"deflect out! %d", g_bouncers[i]->collisionCount);
				}
								
				//NSLog( @"yay! bounce number %d from projectile %d", g_bouncers[i]->collisionCount, i);
				
				if ( g_bouncers[i]->collisionCount >= maxCollisions ) {
					g_bouncers[i]->collisionCount = 0;
					g_bouncers[i]->active = false;
				}
			}
		}		
	}
	
	drawLineBetweenTouches();
	drawEntities();
}

